@@ -15,30 +15,39 @@
 ##
 ## and other examples.
 
+
+#' @useDynLib caret
+#' @import methods plyr reshape2 ggplot2 lattice nlme
+NULL
+
+
+.onUnload <- function(libpath) { library.dynam.unload("caret", libpath) }
+
+
 ###################################################################
 ## Global Variables
 ###################################################################
 
 if(getRversion() >= "2.15.1"){
-  
+
   utils::globalVariables(c('Metric', 'Model'))
-  
-  
+
+
   ## densityplot(~ values|Metric, data = plotData, groups = ind,
   ##             xlab = "", ...)
-  
+
   utils::globalVariables(c('ind'))
-  
+
   ##   avPerf <- ddply(subset(results, Metric == metric[1] & X2 == "Estimate"),
   ##                   .(Model),
   ##                   function(x) c(Median = median(x$value, na.rm = TRUE)))
-  
+
   utils::globalVariables(c('X2'))
-  
+
   ## x[[i]]$resample <- subset(x[[i]]$resample, Variables == x[[i]]$bestSubset)
-  
+
   utils::globalVariables(c('Variables'))
-  
+
   ## calibCalc: no visible binding for global variable 'obs'
   ## calibCalc: no visible binding for global variable 'bin'
   ##
@@ -47,9 +56,9 @@ if(getRversion() >= "2.15.1"){
   ##     binData <-  data.frame(prob = x$calibProbVar,
   ##                            bin = cut(x$calibProbVar, (0:cuts)/cuts, include.lowest = TRUE),
   ##                            class = x$calibClassVar)
-  
+
   utils::globalVariables(c('obs', 'bin'))
-  
+
   ##
   ## checkConditionalX: no visible binding for global variable '.outcome'
   ## checkConditionalX <- function(x, y)
@@ -57,9 +66,9 @@ if(getRversion() >= "2.15.1"){
   ##     x$.outcome <- y
   ##     unique(unlist(dlply(x, .(.outcome), zeroVar)))
   ##   }
-  
+
   utils::globalVariables(c('.outcome'))
-  
+
   ## classLevels.splsda: no visible global function definition for 'ilevels'
   ##
   ## classLevels.splsda <- function(x, ...)
@@ -68,9 +77,9 @@ if(getRversion() >= "2.15.1"){
   ##     ## same class name, but this works for either
   ##     ilevels(x$y)
   ##   }
-  
+
   utils::globalVariables(c('ilevels'))
-  
+
   ## looRfeWorkflow: no visible binding for global variable 'iter'
   ## looSbfWorkflow: no visible binding for global variable 'iter'
   ## looTrainWorkflow: no visible binding for global variable 'parm'
@@ -93,9 +102,9 @@ if(getRversion() >= "2.15.1"){
   ##            .errorhandling = "stop") %dopar%
   ##    {
   ##
-  
+
   utils::globalVariables(c('iter', 'parm', 'method', 'Resample', 'dat'))
-  
+
   ## tuneScheme: no visible binding for global variable '.alpha'
   ## tuneScheme: no visible binding for global variable '.phi'
   ## tuneScheme: no visible binding for global variable '.lambda'
@@ -103,9 +112,9 @@ if(getRversion() >= "2.15.1"){
   ##  seqParam[[i]] <- data.frame(.lambda = subset(grid,
   ##                              subset = .phi == loop$.phi[i] &
   ##                              .lambda < loop$.lambda[i])$.lambda)
-  
+
   utils::globalVariables(c('.alpha', '.phi', '.lambda'))
-  
+
   ##  createGrid : somDims: no visible binding for global variable '.xdim'
   ##  createGrid : somDims: no visible binding for global variable '.ydim'
   ##  createGrid : lvqGrid: no visible binding for global variable '.k'
@@ -114,9 +123,9 @@ if(getRversion() >= "2.15.1"){
   ##       out <- expand.grid(.xdim = 1:x, .ydim = 2:(x+1),
   ##                         .xweight = seq(.5, .9, length = len))
   ##
-  
+
   utils::globalVariables(c('.xdim', '.ydim', '.k', '.size'))
-  
+
   ##  createModel: possible error in rda(trainX, trainY, gamma =
   ##    tuneValue$.gamma, lambda = tuneValue$.lambda, ...): unused
   ##    argument(s) (gamma = tuneValue$.gamma, lambda = tuneValue$.lambda)
@@ -144,54 +153,54 @@ if(getRversion() >= "2.15.1"){
   ##
   ## $lambda
   ## [1] NA
-  
+
   ## predictionFunction: no visible binding for global variable '.alpha'
   ##
   ##  delta <- subset(param, .alpha == uniqueA[i])$.delta
   ##
-  
+
   utils::globalVariables(c('.alpha'))
-  
+
   ## predictors.gbm: no visible binding for global variable 'rel.inf'
   ## predictors.sda: no visible binding for global variable 'varIndex'
   ## predictors.smda: no visible binding for global variable 'varIndex'
   ##
   ##    varUsed <- as.character(subset(relImp, rel.inf != 0)$var)
-  
+
   utils::globalVariables(c('rel.inf', 'varIndex'))
-  
+
   ## plotClassProbs: no visible binding for global variable 'Observed'
   ##
   ## out <- densityplot(form, data = stackProbs, groups = Observed, ...)
-  
+
   utils::globalVariables(c('Observed'))
-  
+
   ## plot.train: no visible binding for global variable 'parameter'
   ##
   ## paramLabs <- subset(modelInfo, parameter %in% params)$label
-  
+
   utils::globalVariables(c('parameter'))
-  
+
   ## plot.rfe: no visible binding for global variable 'Selected'
   ##
   ## out <- xyplot(plotForm, data = results, groups = Selected, panel =  panel.profile, ...)
-  
+
   utils::globalVariables(c('Selected'))
-  
+
   ## icr.formula: no visible binding for global variable 'thresh'
   ##
   ## res <- icr.default(x, y, weights = w, thresh = thresh, ...)
-  
+
   utils::globalVariables(c('thresh', 'probValues', 'min_prob', 'groups', 'trainData', 'j', 'x', '.B'))
-  
+
   utils::globalVariables(c('model_id', 'player1', 'player2', 'playa', 'win1', 'win2', 'name'))
-  
+
   utils::globalVariables(c('object', 'Iter', 'lvls', 'Mean', 'Estimate'))
-  
-  
+
+
   ## parse_sampling: no visible binding for global variable 'sampling_methods'
   utils::globalVariables(c('sampling_methods'))
-  
+
   ## ggplot.calibration: no visible binding for global variable 'midpoint'
   ## ggplot.calibration: no visible binding for global variable 'Percent'
   ## ggplot.calibration: no visible binding for global variable 'Lower'
@@ -204,115 +213,20 @@ if(getRversion() >= "2.15.1"){
 ###################################################################
 altTrainWorkflow <- function(x) x
 
+
+#' @export
 best <- function(x, metric, maximize)
 {
-  
+
   bestIter <- if(maximize) which.max(x[,metric])
   else which.min(x[,metric])
-  
+
   bestIter
 }
 
+#' @export
 defaultSummary <- function(data, lev = NULL, model = NULL)
 {
   if(is.character(data$obs)) data$obs <- factor(data$obs, levels = lev)
   postResample(data[,"pred"], data[,"obs"])
 }
-
-twoClassSummary <- function (data, lev = NULL, model = NULL)
-{
-  if(length(levels(data$obs)) > 2)
-    stop(paste("Your outcome has", length(levels(data$obs)),
-               "levels. The twoClassSummary() function isn't appropriate."))
-  requireNamespaceQuietStop('pROC')
-  if (!all(levels(data[, "pred"]) == levels(data[, "obs"])))
-    stop("levels of observed and predicted data do not match")
-  rocObject <- try(pROC::roc(data$obs, data[, lev[1]], direction = ">"), silent = TRUE)
-  rocAUC <- if(class(rocObject)[1] == "try-error") NA else rocObject$auc
-  out <- c(rocAUC,
-           sensitivity(data[, "pred"], data[, "obs"], lev[1]),
-           specificity(data[, "pred"], data[, "obs"], lev[2]))
-  names(out) <- c("ROC", "Sens", "Spec")
-  out
-}
-
-mnLogLoss <- function(data, lev = NULL, model = NULL){
-  if(is.null(lev)) stop("'lev' cannot be NULL")
-  if(!all(lev %in% colnames(data)))
-    stop("'data' should have columns consistent with 'lev'")
-  if(!all(sort(lev) %in% sort(levels(data$obs))))
-    stop("'data$obs' should have levels consistent with 'lev'")
-  eps <- 1e-15
-  probs <- as.matrix(data[, lev, drop = FALSE])
-  probs[probs > 1 - eps] <- 1 - eps
-  probs[probs < eps] <- eps
-  inds <- match(data$obs, colnames(probs))
-  probs <- probs[cbind(seq_len(nrow(probs)), inds)]
-  c(logLoss = -mean(log(probs), na.rm = TRUE))
-}
-
-multiClassSummary <- function (data, lev = NULL, model = NULL){
-  #Check data
-  if (!all(levels(data[, "pred"]) == levels(data[, "obs"]))) 
-    stop("levels of observed and predicted data do not match")
-  has_class_probs <- all(lev %in% colnames(data))
-  if(has_class_probs) {
-    ## Overall multinomial loss
-    lloss <- mnLogLoss(data = data, lev = lev, model = model)
-    requireNamespaceQuietStop("pROC")  
-    #Calculate custom one-vs-all ROC curves for each class
-    prob_stats <- lapply(levels(data[, "pred"]), 
-                         function(class){
-                           #Grab one-vs-all data for the class
-                           obs  <- ifelse(data[,  "obs"] == class, 1, 0)
-                           prob <- data[,class]
-                           rocObject <- try(pROC::roc(obs, data[,class], direction = "<"), silent = TRUE)
-                           prob_stats <- if (class(rocObject)[1] == "try-error") NA else rocObject$auc
-                           names(prob_stats) <- c('ROC')
-                           return(prob_stats) 
-                         })
-    roc_stats <- mean(unlist(prob_stats))
-  }
-  
-  #Calculate confusion matrix-based statistics
-  CM <- confusionMatrix(data[, "pred"], data[, "obs"])
-  
-  #Aggregate and average class-wise stats
-  #Todo: add weights
-  # RES: support two classes here as well
-  #browser() # Debug
-  if (length(levels(data[, "pred"])) == 2) {
-    class_stats <- CM$byClass
-  } else {
-    class_stats <- colMeans(CM$byClass)
-    names(class_stats) <- paste("Mean", names(class_stats))
-  }
-  
-  # Aggregate overall stats
-  overall_stats <- if(has_class_probs) 
-    c(CM$overall, lloss, ROC = roc_stats) else CM$overall
-  if (length(levels(data[, "pred"])) > 2) 
-    names(overall_stats)[names(overall_stats) == "ROC"] <- "Mean_ROC"
-  
-  
-  # Combine overall with class-wise stats and remove some stats we don't want 
-  stats <- c(overall_stats, class_stats)
-  stats <- stats[! names(stats) %in% c('AccuracyNull', "AccuracyLower", "AccuracyUpper",
-                                       "AccuracyPValue", "McnemarPValue", 
-                                       'Mean Prevalence', 'Mean Detection Prevalence')]
-  
-  # Clean names
-  names(stats) <- gsub('[[:blank:]]+', '_', names(stats))
-  
-  # Change name ordering to place most useful first
-  # May want to remove some of these eventually
-  stat_list <- c("Accuracy", "Kappa", "Mean_Sensitivity", "Mean_Specificity", 
-                 "Mean_Pos_Pred_Value", "Mean_Neg_Pred_Value", "Mean_Detection_Rate",
-                 "Mean_Balanced_Accuracy")
-  if(has_class_probs) stat_list <- c("logLoss", "Mean_ROC", stat_list)
-  if (length(levels(data[, "pred"])) == 2) stat_list <- gsub("^Mean_", "", stat_list)
-  
-  stats <- stats[c(stat_list)]
-  
-  return(stats)
-}
