@@ -572,7 +572,8 @@ resampName <- function(x, numbers = TRUE){
                                     ifelse(x$control$fixedWindow, " a ", " no "),
                                     "fixed window)", sep = ""),
                   oob = "Out of Bag Resampling",
-                  boot =, boot632 = paste("Bootstrapped (", numResamp, " reps)", sep = ""),
+                  boot =, optimism_boot =, boot_all =,
+                  boot632 = paste("Bootstrapped (", numResamp, " reps)", sep = ""),
                   cv = paste("Cross-Validated (", x$control$number, " fold)", sep = ""),
                   repeatedcv = paste("Cross-Validated (", x$control$number, " fold, repeated ",
                                      x$control$repeats, " times)", sep = ""),
@@ -593,6 +594,8 @@ resampName <- function(x, numbers = TRUE){
                   timeslice = "Rolling Forecasting Origin Resampling",
                   oob = "Out of Bag Resampling",
                   boot = "(Bootstrap)",
+                  optimism_boot = "(Optimism Bootstrap)",
+                  boot_all = "(Bootstrap All)",
                   boot632 = "(Bootstrap 632 Rule)",
                   cv = "(Cross-Validation)",
                   repeatedcv = "(Repeated Cross-Validation)",
