@@ -132,7 +132,7 @@ stringFunc <- function (x)  {
 
       cat("Resampling results")
       if(dim(tuneAcc)[1] > 1) cat(" across tuning parameters")
-      if(showSD) cat(" (values above are 'mean (sd)')")
+      if(showSD) cat(" (values below are 'mean (sd)')")
       cat(":\n\n")
 
       if(dim(tuneAcc)[1] > 1) {
@@ -199,7 +199,7 @@ stringFunc <- function (x)  {
         } else constString <- NULL
       } else constString <- NULL
 
-      tuneAcc <- tuneAcc[,!grepl("Apparent$", names(tuneAcc)),drop = FALSE]
+      tuneAcc <- tuneAcc[,!grepl("Apparent$|Optimism$", names(tuneAcc)), drop = FALSE]
       colnames(tuneAcc)[colnames(tuneAcc) == ".B"] <- "Resamples"
       nms <- names(tuneAcc)[names(tuneAcc) %in% params]
       sort_args <- vector(mode = "list", length = length(nms))
@@ -264,7 +264,8 @@ stringFunc <- function (x)  {
         cat(truncateText(met))
       }
 
-      cat(truncateText(optString), "\n")
+      cat(truncateText(optString))
+      if(nzchar(optString)) cat("\n")
     } else printMat <- NULL
 
     if(details) {
