@@ -9,6 +9,9 @@ getOper <- function(x) if(x)  `%dopar%` else  `%do%`
 getTrainOper <- function(x) if(x)  `%dopar%` else  `%do%`
 
 
+#' @rdname caret-internal
+#' @export
+#' @keywords internal
 progress <- function(x, names, iter, start = TRUE)
 {
   text <- paste(ifelse(start, "+ ", "- "),
@@ -18,6 +21,9 @@ progress <- function(x, names, iter, start = TRUE)
   cat(text, "\n")
 }
 
+#' @rdname caret-internal
+#' @importFrom stats sd
+#' @export
 MeanSD <- function(x, exclude = NULL)
 {
   if(!is.null(exclude)) x <- x[, !(colnames(x) %in% exclude), drop = FALSE]
@@ -26,6 +32,8 @@ MeanSD <- function(x, exclude = NULL)
   out
 }
 
+#' @rdname caret-internal
+#' @export
 expandParameters <- function(fixed, seq)
 {
   if(is.null(seq)) return(fixed)
@@ -41,6 +49,9 @@ expandParameters <- function(fixed, seq)
   out
 }
 
+#' @importFrom utils head
+#' @importFrom stats complete.cases
+#' @import foreach
 nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing = FALSE, ...)
 {
   loadNamespace("caret")
@@ -54,10 +65,11 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
   ## fitting and predicting the full data set.
   
   resampleIndex <- ctrl$index
-  if(ctrl$method %in% c("boot632"))
+  if(ctrl$method %in% c("boot632", "optimism_boot", "boot_all"))
   {
     resampleIndex <- c(list("AllData" = rep(0, nrow(x))), resampleIndex)
     ctrl$indexOut <- c(list("AllData" = rep(0, nrow(x))),  ctrl$indexOut)
+    if(!is.null(ctrl$indexExtra)) ctrl$indexExtra <- c(list("AllData" = NULL), ctrl$indexExtra)
   }
   `%op%` <- getOper(ctrl$allowParallel && getDoParWorkers() > 1)
   keep_pred <- isTRUE(ctrl$savePredictions) || ctrl$savePredictions %in% c("all", "final")
@@ -83,6 +95,8 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
         holdoutIndex <- modelIndex
       }
       
+      extraIndex <- if(is.null(ctrl$indexExtra)) NULL else ctrl$indexExtra[[iter]]
+      
       if(testing) cat("pre-model\n")
 
       if(!is.null(info$submodels[[parm]]) && nrow(info$submodels[[parm]]) > 0) {
@@ -90,7 +104,7 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
       } else submod <- NULL
       
       mod <- try(
-        createModel(x = x[modelIndex,,drop = FALSE ],
+        createModel(x = subset_x(x, modelIndex),
                     y = y[modelIndex],
                     wts = wts[modelIndex],
                     method = method,
@@ -104,12 +118,13 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
       
       if(testing) print(mod) 
       
+      predictedExtra <- NULL
       if(class(mod)[1] != "try-error")
       {
         predicted <- try(
           predictionFunction(method = method,
                              modelFit = mod$fit,
-                             newdata = x[holdoutIndex,, drop = FALSE],
+                             newdata = subset_x(x, holdoutIndex),
                              preProc = mod$preProc,
                              param = submod),
           silent = TRUE)
@@ -142,6 +157,14 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
             for(i in seq(along = predicted)) predicted[[i]] <- tmp
             rm(tmp)
           }
+        } else if(!is.null(extraIndex)) {
+          predictedExtra <- lapply(extraIndex, function(idx) {
+            predictionFunction(method = method,
+                               modelFit = mod$fit,
+                               newdata = subset_x(x, idx),
+                               preProc = mod$preProc,
+                               param = submod)
+          })
         }
       } else {
         wrn <- paste(colnames(printed[parm,,drop = FALSE]),
@@ -173,29 +196,45 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
       }
       
       if(testing) print(head(predicted))
+      probValuesExtra <- NULL
       if(ctrl$classProbs)
       {
         if(class(mod)[1] != "try-error")
         {
           probValues <- probFunction(method = method,
                                      modelFit = mod$fit,
-                                     newdata = x[holdoutIndex,, drop = FALSE],
+                                     newdata = subset_x(x, holdoutIndex),
                                      preProc = mod$preProc,
                                      param = submod)
+          
+          if (!is.null(extraIndex))
+            probValuesExtra <- lapply(extraIndex, function(index) {
+              probFunction(method = method,
+                           modelFit = mod$fit,
+                           newdata = subset_x(x, index),
+                           preProc = mod$preProc,
+                           param = submod)
+            })
         } else {
           probValues <- as.data.frame(matrix(NA, nrow = nPred, ncol = length(lev)))
           colnames(probValues) <- lev
           if(!is.null(submod))
           {
-            tmp <- probValues
-            probValues <- vector(mode = "list", length = nrow(info$submodels[[parm]]) + 1)
-            for(i in seq(along = probValues)) probValues[[i]] <- tmp
-            rm(tmp)
+            probValues <- rep(list(probValues), nrow(info$submodels[[parm]]) + 1L)
           }
         }
         if(testing) print(head(probValues))
       }
       
+      if(is.null(probValuesExtra)) {
+        probValuesExtra <- as.data.frame(matrix(NA, nrow = nrow(x), ncol = length(lev)))
+        colnames(probValuesExtra) <- lev
+        if(!is.null(submod)) {
+          probValuesExtra <- rep(list(probValuesExtra), nrow(info$submodels[[parm]]) + 1L)
+        }
+        probValuesExtra <- rep(list(probValuesExtra), 2L)
+      }
+      
       ##################################
       
       if(is.numeric(y)) {
@@ -237,7 +276,7 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
         ## collate the predicitons across all the sub-models
         predicted <- lapply(predicted,
                             function(x, y, wts, lv, rows) {
-                              x <- outcome_conversion(x, lv = lev)
+                              x <- getFromNamespace("outcome_conversion", "caret")(x, lv = lev)
                               out <- data.frame(pred = x, obs = y, stringsAsFactors = FALSE)
                               if(!is.null(wts)) out$weights <- wts
                               out$rowIndex <- rows
@@ -247,20 +286,39 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
                             wts = wts[holdoutIndex],
                             lv = lev,
                             rows = holdoutIndex)
+        if(!is.null(predictedExtra))
+          predictedExtra <- mapply(predictedExtra, extraIndex, 
+                                   SIMPLIFY = FALSE, USE.NAMES = FALSE,
+                                   FUN = function(pred, rows) {
+                                     lapply(pred, function(x) {
+                                       y <- y[rows]
+                                       wts <- wts[rows]
+                                       
+                                       x <- outcome_conversion(x, lv = lev)
+                                       out <- data.frame(pred = x, obs = y, stringsAsFactors = FALSE)
+                                       if(!is.null(wts)) out$weights <- wts
+                                       out$rowIndex <- rows
+                                       out
+                                     })
+                                   })
         if(testing) print(head(predicted))
         
         ## same for the class probabilities
-        if(ctrl$classProbs)
-        {
-          for(k in seq(along = predicted)) predicted[[k]] <- cbind(predicted[[k]], probValues[[k]])
+        if(ctrl$classProbs) {
+          predicted <- mapply(cbind, predicted, probValues, SIMPLIFY = FALSE)
+          if (!is.null(predictedExtra))
+            predictedExtra <- mapply(predictedExtra, probValuesExtra,
+                                     SIMPLIFY = FALSE,
+                                     FUN = function(predEx, probEx) {
+                                       mapply(cbind, predEx, probEx, SIMPLIFY = FALSE)
+                                     })
         }
         
-        if(keep_pred)
+        if(keep_pred || (ctrl$method == "boot_all" && names(resampleIndex)[iter] == "AllData"))
         {
           tmpPred <- predicted
           for(modIndex in seq(along = tmpPred))
           {
-            tmpPred[[modIndex]]$rowIndex <- holdoutIndex
             tmpPred[[modIndex]] <- merge(tmpPred[[modIndex]], 
                                          allParam[modIndex,,drop = FALSE],
                                          all = TRUE)
@@ -275,6 +333,26 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
                                lev = lev,
                                model = method)
         if(testing) print(head(thisResample))
+        
+        if(!is.null(predictedExtra)) {
+          thisResampleExtra <- lapply(predictedExtra, function(predicted) {
+            lapply(predicted,
+                   ctrl$summaryFunction,
+                   lev = lev,
+                   model = method)
+          })
+          thisResampleExtra[[1L]] <- lapply(thisResampleExtra[[1L]], function(res) {
+            names(res) <- paste0(names(res), "Orig")
+            res
+          })
+          thisResampleExtra[[2L]] <- lapply(thisResampleExtra[[2L]], function(res) {
+            names(res) <- paste0(names(res), "Boot")
+            res
+          })
+          thisResampleExtra <- do.call(cbind, lapply(thisResampleExtra, function(x) do.call(rbind, x)))
+          thisResampleExtra <- cbind(allParam, thisResampleExtra)
+        } else thisResampleExtra <- NULL
+        
         ## for classification, add the cell counts
         if(length(lev) > 1 && length(lev) <= 50)
         {
@@ -286,7 +364,7 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
         thisResample <- cbind(allParam, thisResample)
         
       } else {       
-        if(is.factor(y)) predicted <- outcome_conversion(predicted, lv = lev)
+        if(is.factor(y)) predicted <- getFromNamespace("outcome_conversion", "caret")(predicted, lv = lev)
         tmp <-  data.frame(pred = predicted,
                            obs = y[holdoutIndex],
                            stringsAsFactors = FALSE)
@@ -297,7 +375,7 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
         if(ctrl$classProbs) tmp <- cbind(tmp, probValues)
         tmp$rowIndex <- holdoutIndex
         
-        if(keep_pred)
+        if(keep_pred || (ctrl$method == "boot_all" && names(resampleIndex)[iter] == "AllData"))
         {
           tmpPred <- tmp
           tmpPred$rowIndex <- holdoutIndex
@@ -317,6 +395,33 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
         thisResample <- as.data.frame(t(thisResample))
         thisResample <- cbind(thisResample, info$loop[parm,,drop = FALSE])
         
+        ## for optimism bootstrap
+        if(!is.null(predictedExtra)) {
+          thisResampleExtra <- mapply(predictedExtra, extraIndex, probValuesExtra,
+                                      SIMPLIFY = FALSE, USE.NAMES = FALSE,
+                                      FUN = function(predicted, holdoutIndex, probValues) {
+                                        if(is.factor(y)) predicted <- outcome_conversion(predicted, lv = lev)
+                                        tmp <-  data.frame(pred = predicted,
+                                                           obs = y[holdoutIndex],
+                                                           stringsAsFactors = FALSE)
+                                        ## Sometimes the code above does not coerce the first
+                                        ## columnn to be named "pred" so force it
+                                        names(tmp)[1] <- "pred"
+                                        if(!is.null(wts)) tmp$weights <- wts[holdoutIndex]
+                                        if(ctrl$classProbs) tmp <- cbind(tmp, probValues)
+                                        tmp$rowIndex <- holdoutIndex
+                                        ctrl$summaryFunction(tmp, lev = lev, model = method)
+                                      })
+          
+          names(thisResampleExtra[[1L]]) <- paste0(names(thisResampleExtra[[1L]]), "Orig")
+          names(thisResampleExtra[[2L]]) <- paste0(names(thisResampleExtra[[2L]]), "Boot")
+          
+          thisResampleExtra <- unlist(thisResampleExtra, recursive = FALSE)
+          
+          thisResampleExtra <- cbind(as.data.frame(t(thisResampleExtra)), info$loop[parm, , drop = FALSE])
+          
+        } else thisResampleExtra <- NULL
+        
       }
       thisResample$Resample <- names(resampleIndex)[iter]
       
@@ -324,12 +429,13 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
                                     names(resampleIndex), iter, FALSE)
       
       if(testing) print(thisResample)
-      list(resamples = thisResample, pred = tmpPred)
+      list(resamples = thisResample, pred = tmpPred, resamplesExtra = thisResampleExtra)
     }
   
   resamples <- rbind.fill(result[names(result) == "resamples"])
-  pred <- if(keep_pred)  rbind.fill(result[names(result) == "pred"]) else NULL
-  if(ctrl$method %in% c("boot632"))
+  pred <- rbind.fill(result[names(result) == "pred"])
+  resamplesExtra <- rbind.fill(result[names(result) == "resamplesExtra"])
+  if(ctrl$method %in% c("boot632", "optimism_boot", "boot_all"))
   {
     perfNames <- names(resamples)
     perfNames <- perfNames[!(perfNames %in% c("Resample", as.character(method$parameters$parameter)))]
@@ -344,6 +450,11 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
       warning("There were missing values in the apparent performance measures.")
     }        
     resamples <- subset(resamples, Resample != "AllData")
+    if(!is.null(pred))
+    {
+      predHat <- subset(pred, Resample == "AllData")
+      pred <- subset(pred, Resample != "AllData")
+    }
   }
   names(resamples) <- gsub("^\\.", "", names(resamples))
   
@@ -358,19 +469,43 @@ nominalTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, tes
                MeanSD, 
                exclude = gsub("^\\.", "", colnames(info$loop)))
   
-  if(ctrl$method %in% c("boot632")) {
+  if(ctrl$method %in% c("boot632", "boot_all")) {
     out <- merge(out, apparent)
-    for(p in seq(along = perfNames)) {
-      const <- 1-exp(-1)
-      out[, perfNames[p]] <- (const * out[, perfNames[p]]) +  ((1-const) * out[, paste(perfNames[p],"Apparent", sep = "")])
-    }
+    const <- 1 - exp(-1)
+    sapply(perfNames, function(perfName) {
+      perfOut <- if(ctrl$method == "boot_all") paste0(perfName, "_632") else perfName
+      out[, perfOut] <<- (const * out[, perfName]) +  ((1-const) * out[, paste(perfName, "Apparent", sep = "")])
+      NULL
+    })
+  }
+  
+  if(ctrl$method %in% c("optimism_boot", "boot_all")) {
+    out <- merge(out, apparent)
+    out <- merge(out, ddply(resamplesExtra[, !grepl("Resample", colnames(resamplesExtra)), drop = FALSE],
+                            colnames(info$loop),
+                            function(df, exclude) {
+                              colMeans(df[, setdiff(colnames(df), exclude), drop = FALSE])
+                            },
+                            exclude = colnames(info$loop)))
+    sapply(perfNames, function(perfName) {
+      optimism <- out[ , paste0(perfName, "Orig")] - out[ , paste0(perfName, "Boot")]
+      final_estimate <- out[ , paste0(perfName, "Apparent")] + optimism
+      ## Remove unnecessary values
+      out[ , paste0(perfName, "Orig")] <<- NULL
+      out[ , paste0(perfName, "Boot")] <<- NULL
+      perfOut <- if(ctrl$method == "boot_all") paste0(perfName, "_OptBoot") else perfName
+      ## Update estimates
+      out[ , paste0(perfName, "Optimism")] <<- optimism
+      out[ , perfOut] <<- final_estimate
+      NULL
+    })
   }
   
   list(performance = out, resamples = resamples, predictions = if(keep_pred) pred else NULL)
 }
 
 
-
+#' @import foreach
 looTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing = FALSE, ...)
 {
   loadNamespace("caret")
@@ -398,7 +533,7 @@ looTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing
         submod <- info$submodels[[parm]]
       } else submod <- NULL
       
-      mod <- createModel(x = x[ctrl$index[[iter]],,drop = FALSE ],
+      mod <- createModel(x = subset_x(x, ctrl$index[[iter]]),
                          y = y[ctrl$index[[iter]] ],
                          wts = wts[ctrl$index[[iter]] ],
                          method = method,
@@ -413,7 +548,7 @@ looTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing
       
       predicted <- predictionFunction(method = method,
                                       modelFit = mod$fit,
-                                      newdata = x[-ctrl$index[[iter]],, drop = FALSE],
+                                      newdata = subset_x(x, -ctrl$index[[iter]]),
                                       preProc = mod$preProc,
                                       param = submod)
       
@@ -452,7 +587,7 @@ looTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing
       {
         probValues <- probFunction(method = method,
                                    modelFit = mod$fit,
-                                   newdata = x[holdoutIndex,, drop = FALSE],
+                                   newdata = subset_x(x, holdoutIndex),
                                    preProc = mod$preProc,
                                    param = submod)
         if(testing) print(head(probValues))
@@ -465,7 +600,7 @@ looTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing
         ## collate the predictions across all the sub-models
         predicted <- lapply(predicted,
                             function(x, y, wts, lv, rows) {
-                              x <- outcome_conversion(x, lv = lev)
+                              x <- getFromNamespace("outcome_conversion", "caret")(x, lv = lev)
                               out <- data.frame(pred = x, obs = y, stringsAsFactors = FALSE)
                               if(!is.null(wts)) out$weights <- wts
                               out$rowIndex <- rows
@@ -488,7 +623,7 @@ looTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing
         predicted <- cbind(predicted, allParam)
         ## if saveDetails then save and export 'predicted'
       } else {
-        predicted <- outcome_conversion(predicted, lv = lev)
+        predicted <- getFromNamespace("outcome_conversion", "caret")(predicted, lv = lev)
         predicted <-  data.frame(pred = predicted,
                                  obs = y[holdoutIndex],
                                  stringsAsFactors = FALSE)
@@ -512,6 +647,7 @@ looTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing
   list(performance = out, predictions = result)
 }
 
+#' @import foreach
 oobTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing = FALSE, ...)
 {
   loadNamespace("caret")
@@ -552,6 +688,7 @@ oobTrainWorkflow <- function(x, y, wts, info, method, ppOpts, ctrl, lev, testing
 
 ################################################################################################
 
+#' @import foreach
 nominalSbfWorkflow <- function(x, y, ppOpts, ctrl, lev, ...)
 {
   loadNamespace("caret")
@@ -579,9 +716,9 @@ nominalSbfWorkflow <- function(x, y, ppOpts, ctrl, lev, ...)
       holdoutIndex <- modelIndex
     }
     
-    sbfResults <- sbfIter(x[modelIndex,,drop = FALSE],
+    sbfResults <- sbfIter(subset_x(x, modelIndex),
                           y[modelIndex],
-                          x[holdoutIndex,,drop = FALSE],
+                          subset_x(x, holdoutIndex),
                           y[holdoutIndex],
                           ctrl,
                           ...)
@@ -607,9 +744,9 @@ nominalSbfWorkflow <- function(x, y, ppOpts, ctrl, lev, ...)
   {
     modelIndex <- 1:nrow(x)
     holdoutIndex <- modelIndex
-    appResults <- sbfIter(x[modelIndex,,drop = FALSE],
+    appResults <- sbfIter(subset_x(x, modelIndex),
                           y[modelIndex],
-                          x[holdoutIndex,,drop = FALSE],
+                          subset_x(x, holdoutIndex),
                           y[holdoutIndex],
                           ctrl,
                           ...)
@@ -627,7 +764,7 @@ nominalSbfWorkflow <- function(x, y, ppOpts, ctrl, lev, ...)
   list(performance = performance, everything = result, predictions = if(ctrl$saveDetails) pred else NULL)
 }
 
-
+#' @import foreach
 looSbfWorkflow <- function(x, y, ppOpts, ctrl, lev, ...)
 {
   loadNamespace("caret")
@@ -648,9 +785,9 @@ looSbfWorkflow <- function(x, y, ppOpts, ctrl, lev, ...)
     modelIndex <- resampleIndex[[iter]]
     holdoutIndex <- -unique(resampleIndex[[iter]])
     
-    sbfResults <- sbfIter(x[modelIndex,,drop = FALSE],
+    sbfResults <- sbfIter(subset_x(x, modelIndex),
                           y[modelIndex],
-                          x[holdoutIndex,,drop = FALSE],
+                          subset_x(x, holdoutIndex),
                           y[holdoutIndex],
                           ctrl,
                           ...)
@@ -666,6 +803,7 @@ looSbfWorkflow <- function(x, y, ppOpts, ctrl, lev, ...)
 
 ################################################################################################
 
+#' @import foreach
 nominalRfeWorkflow <- function(x, y, sizes, ppOpts, ctrl, lev, ...)
 {
   loadNamespace("caret")
@@ -692,9 +830,9 @@ nominalRfeWorkflow <- function(x, y, sizes, ppOpts, ctrl, lev, ...)
     }
     
     seeds <- if(!(length(ctrl$seeds) == 1 && is.na(ctrl$seeds))) ctrl$seeds[[iter]] else NA
-    rfeResults <- rfeIter(x[modelIndex,,drop = FALSE],
+    rfeResults <- rfeIter(subset_x(x, modelIndex),
                           y[modelIndex],
-                          x[holdoutIndex,,drop = FALSE],
+                          subset_x(x, holdoutIndex),
                           y[holdoutIndex],
                           sizes,
                           ctrl,
@@ -755,7 +893,7 @@ nominalRfeWorkflow <- function(x, y, sizes, ppOpts, ctrl, lev, ...)
   list(performance = externPerf, everything = result)
 }
 
-
+#' @import foreach
 looRfeWorkflow <- function(x, y, sizes, ppOpts, ctrl, lev, ...)
 {
   loadNamespace("caret")
@@ -772,9 +910,9 @@ looRfeWorkflow <- function(x, y, sizes, ppOpts, ctrl, lev, ...)
     holdoutIndex <- -unique(resampleIndex[[iter]])
     
     seeds <- if(!(length(ctrl$seeds) == 1 && is.na(ctrl$seeds))) ctrl$seeds[[iter]] else NA
-    rfeResults <- rfeIter(x[modelIndex,,drop = FALSE],
+    rfeResults <- rfeIter(subset_x(x, modelIndex),
                           y[modelIndex],
-                          x[holdoutIndex,,drop = FALSE],
+                          subset_x(x, holdoutIndex),
                           y[holdoutIndex],
                           sizes,
                           ctrl,
